#include <proto.h>

#include "gtpicture.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtPicture class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

#define GTPT_PIXMAP		0
#define GTPT_IMAGE		1

/************************************************************************\
*
* Create and init picture.
*
\************************************************************************/
GtPicture *GtMkPicture(char *n,GtManager *p,GtTag *tgs)
{
	GtPicture *obj;
	
	if(!(obj=new GtPicture()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtPicture object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init picture.
*
\************************************************************************/
GtPicture *GtMkPicture(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtPicture *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkPicture(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty picture.
*
\************************************************************************/
GtPicture::GtPicture(void)
{
	ptype=GTPT_PIXMAP;
	pd.pix=NULL;
	autoref=TRUE;
	pw=1;
	ph=1;
}

/************************************************************************\
*
* Setup picture.
*
\************************************************************************/
sbool GtPicture::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	long xs,ys;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Pixmap:
				pd.pix=(GtPixmap *)t->d.ptr;
				ptype=GTPT_PIXMAP;
				break;
			case GT_Image:
				pd.im=(GtImage *)t->d.ptr;
				ptype=GTPT_IMAGE;
				break;
			case GT_AutoRefresh:
				if(t->d.val)
					autoref=TRUE;
				else
					autoref=FALSE;
				break;
		}
	}
	if((ptype==GTPT_PIXMAP) && pd.pix)
	{
		pd.pix->GetV(GT_XSize,&xs,GT_YSize,&ys,GT_DONE);
		pw=xs;
		ph=ys;
	}
	else if((ptype==GTPT_IMAGE) && pd.im)
	{
		pd.im->GetV(GT_XSize,&xs,GT_YSize,&ys,GT_DONE);
		pw=xs;
		ph=ys;
	}
	this->SetMinMaxSize(pw,ph,pw,ph);
	return(TRUE);
}
	
/************************************************************************\
*
* Set picture attributes.
*
\************************************************************************/
sbool GtPicture::Set(GtTag *tgs)
{
	GtTag *t;
	long xs,ys;
	sbool ch=FALSE;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Pixmap:
				pd.pix=(GtPixmap *)t->d.ptr;
				ptype=GTPT_PIXMAP;
				ch=TRUE;
				break;
			case GT_Image:
				pd.im=(GtImage *)t->d.ptr;
				ptype=GTPT_IMAGE;
				ch=TRUE;
				break;
			case GT_AutoRefresh:
				if(t->d.val)
					autoref=TRUE;
				else
					autoref=FALSE;
				break;
		}
	}
	if(ch)
	{
		if((ptype==GTPT_PIXMAP) && pd.pix)
		{
			pd.pix->GetV(GT_XSize,&xs,GT_YSize,&ys,GT_DONE);
			pw=xs;
			ph=ys;
		}
		else if((ptype==GTPT_IMAGE) && pd.im)
		{
			pd.im->GetV(GT_XSize,&xs,GT_YSize,&ys,GT_DONE);
			pw=xs;
			ph=ys;
		}
		this->SetMinMaxSize(pw,ph,pw,ph);
		this->Refresh();
	}
	return(TRUE);
}
	
/************************************************************************\
*
* Get picture attributes.
*
\************************************************************************/
void GtPicture::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Pixmap:
				if(ptype==GTPT_PIXMAP)
					*ptr=(long)pd.pix;
				break;
			case GT_Image:
				if(ptype==GTPT_IMAGE)
					*ptr=(long)pd.im;
				break;
			case GT_AutoRefresh:
				*ptr=autoref;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtPicture::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtPicture")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Refresh picture.
*
\************************************************************************/
void GtPicture::Refresh(void)
{
	if(gfx)
	{
		gtwin->Clear();
		this->ReDraw(0,0,pw,ph);
	}
}

/************************************************************************\
*
* Refresh part of the picture.
*
\************************************************************************/
void GtPicture::Refresh(int xp,int yp,int xs,int ys)
{
	if(gfx)
	{
		gtwin->Clear(xp,yp,xs,ys);
		this->ReDraw(xp,yp,xs,ys);
	}
}

/************************************************************************\
*
* Set picture event mask.
*
\************************************************************************/
void GtPicture::SetEMask(ulong em)
{
	GtWidget::SetEMask(em|ExposureMask);
}

/************************************************************************\
*
* Draw picture.
*
\************************************************************************/
sbool GtPicture::Draw(XEvent *xe)
{
	if(xe && (xe->type==Expose) && autoref)
		this->ReDraw(xe->xexpose.x,xe->xexpose.y,
									xe->xexpose.width,xe->xexpose.height);
	return(TRUE);
}

/************************************************************************\
*
* Redraw part of the picture.
*
\************************************************************************/
void GtPicture::ReDraw(int xp,int yp,int xs,int ys)
{
	xp=GTMAX(xp,0);
	yp=GTMAX(yp,0);
	xs=GTMIN(xs,pw-xp);
	ys=GTMIN(ys,ph-yp);
	if(ptype==GTPT_PIXMAP)
	{
		if(pd.pix)
		{
			if(pd.pix->IsBitmap())
				gtfore->CopyPlane(pd.pix,gtwin,xp,yp,xs,ys,xp,yp,1);
			else
				gtfore->CopyArea(pd.pix,gtwin,xp,yp,xs,ys,xp,yp);
		}
	}
	else
	{
		if(pd.im)
			pd.im->PutImage(gtwin,gtfore,xp,yp,xp,yp,xs,ys);
	}
}
